/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.aou;

import it.tidalwave.role.LocalizedDisplayable;
import it.tidalwave.role.Displayable;
import javax.annotation.Nonnull;
import java.util.Scanner;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import org.openrdf.repository.Repository;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.birds.BirdTaxonomyImporter;
import static it.tidalwave.bluebill.taxonomy.elmo.ElmoTaxonomyVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class Aou7thImporter extends BirdTaxonomyImporter
  {
    private static final String CLASS = Aou7thImporter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private static final String[] LANGUAGES = new String[] { "en_us", "fr_ca"};

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxonomy run (final @Nonnull Repository repository,
                         final @Nonnull String taxonomyName)
      throws Exception
      {
        init(repository);

        final String idPrefix = ID_TAXONOMY_PREFIX + "AOU/7th";
        final Taxonomy taxonomy = createTaxonomy(repository, idPrefix, Aou7thImporter.class, LANGUAGES);
        final Taxon aves = createAves(taxonomy, idPrefix);

        final InputStream is = getClass().getResourceAsStream("AOUlist.csv");
        final BufferedReader br = new BufferedReader(new InputStreamReader(is, "iso-8859-1"));

        br.readLine();
        br.readLine();

        for (;;)
          {
            final String s = br.readLine();

            if (s == null)
              {
                break;
              }

            System.err.println(s);
            final Scanner scanner = new Scanner(s).useDelimiter(",");
            final String scientificName = dequoted(scanner.next());
            String englishName = dequoted(scanner.next());
            String frenchName = dequoted(scanner.next());
            final String orderName = dequoted(scanner.next());
            final String familyName = dequoted(scanner.next());

            final Scanner scanner2 = new Scanner(scientificName).useDelimiter(" ");
            final String genusName = scanner2.next();
            final String speciesName = scanner2.next();
            final String subSpeciesName = scanner2.hasNext() ? scanner2.next() : null;

            logger.info("%s %s %s %s %s %s %s %s", scientificName, englishName, frenchName, orderName, familyName, genusName, speciesName, subSpeciesName);

            if (orderName.contains("Incertae Sedis") || familyName.contains("Incertae Sedis"))
              {
                continue;
              }

            final LocalizedDisplayable commonNames = createDisplayable("en_us", englishName,
                                                                      "fr_ca", frenchName);
            createPath(aves, idPrefix, orderName, familyName, genusName, speciesName, null, NO_COMMON_NAMES, commonNames);
          }

        br.close();
        taxonUniqueIdManager.close();

        return taxonomy;
      }

    @Nonnull
    protected String dequoted (final @Nonnull String string)
      {
        return string.replaceAll("^\"", "").replaceAll("\"$", "");
      }
  }
